@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: CmdExec.cc,v 1.115 2005/08/12 07:05:32 lav Exp $ */
+/* $Id: CmdExec.cc,v 1.117 2005/12/08 13:50:06 lav Exp $ */
 
 #include <config.h>
 #include <errno.h>
@@ -700,7 +700,7 @@ void CmdExec::ShowRunStatus(StatusLine *s)
    {
    case(BUILTIN_CD):
       if(session->IsOpen())
-	 s->Show("cd `%s' [%s]",args->getarg(1),session->CurrentStatus());
+	 s->Show("cd `%s' [%s]",squeeze_file_name(args->getarg(1),s->GetWidthDelayed()-40),session->CurrentStatus());
       break;
    case(BUILTIN_OPEN):
       if(session->IsOpen())
@@ -734,6 +734,7 @@ void CmdExec::PrintStatus(int v,const char *prefix)
    {
       if(IsSuspended())
 	 printf("%s%s\n",prefix,_("Queue is stopped."));
+      BuryDoneJobs();
       for(int i=0; i<waiting_num; i++)
       {
 	 if(i==0)
