@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: CmdExec.h,v 1.52 2004/10/18 07:10:48 lav Exp $ */
+/* $Id: CmdExec.h,v 1.54 2005/08/03 07:12:44 lav Exp $ */
 
 #ifndef CMDEXEC_H
 #define CMDEXEC_H
@@ -52,6 +52,9 @@ public:
 
    virtual void clear() {}
    virtual bool RealEOF() { return true; }
+
+   virtual void Fg() {}
+   virtual void Bg() {}
 };
 
 extern CmdFeeder *lftp_feeder;	 // feeder to use after 'lftp' command
@@ -121,7 +124,7 @@ private:
    }
       builtin;
 
-   char *old_cwd;
+   FileAccess::Path old_cwd;
    char *old_lcwd;
    char *slot;
 
