@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileCopy.cc,v 1.115 2005/06/22 08:04:17 lav Exp $ */
+/* $Id: FileCopy.cc,v 1.118 2005/12/27 12:16:58 lav Exp $ */
 
 /* FileCopyPeer behaviour:
     1) when suspended, does nothing
@@ -147,8 +147,10 @@ int FileCopy::Do()
 	 SetError(put->ErrorText());
 	 return MOVED;
       }
-      if(get->Error())
+      if(get->Error() && get->Size()==0)
       {
+	 put->PutEOF();
+	 Roll(put);
       get_error:
 	 SetError(get->ErrorText());
 	 return MOVED;
@@ -284,14 +286,15 @@ int FileCopy::Do()
 
 	 // now find eol in line_buffer.
 	 line_buffer->Get(&lb,&ls);
-	 while(ls>0)
+	 const char *eol=0;
+	 if(get->Eof() || get->Error())
+	    eol=lb+ls-1;
+	 else
+	    eol=memrchr(lb,'\n',ls);
+	 if(eol)
 	 {
-	    const char *eol=(const char *)memchr(lb,'\n',ls);
-	    if(!eol)
-	       break;
 	    put->Put(lb,eol-lb+1);
 	    line_buffer->Skip(eol-lb+1);
-	    line_buffer->Get(&lb,&ls);
 	 }
       }
       else
@@ -321,6 +324,7 @@ int FileCopy::Do()
 	 line_buffer->Skip(s);
       }
    pre_CONFIRM_WAIT:
+      put->SetSuggestedFileName(get->GetSuggestedFileName());
       put->SetDate(get->GetDate());
       if(get->GetSize()!=NO_SIZE && get->GetSize()!=NO_SIZE_YET)
 	 put->SetEntitySize(get->GetSize());
@@ -684,10 +688,13 @@ FileCopyPeer::FileCopyPeer(dir_t m) : IOBuffer(m)
    use_cache=true;
    write_allowed=true;
    done=false;
+   suggested_filename=0;
+   auto_rename=false;
    Suspend();  // don't do anything too early
 }
 FileCopyPeer::~FileCopyPeer()
 {
+   xfree(suggested_filename);
 }
 
 // FileCopyPeerFA implementation
@@ -1076,6 +1083,7 @@ int FileCopyPeerFA::Get_LL(int len)
    {
       eof=true;
       LsCache::Add(session,file,FAmode,FA::OK,this);
+      SetSuggestedFileName(session->GetSuggestedFileName());
    }
    return res;
 }
@@ -1338,6 +1346,14 @@ int FileCopyPeerFDStream::Do()
 	 SetError(verify->ErrorText());
       if(verify->Done())
       {
+	 if(suggested_filename && stream && stream->full_name && auto_rename)
+	 {
+	    char *dir=alloca_strdup(stream->full_name);
+	    dirname_modify(dir);
+	    debug((5,"copy: renaming `%s' to `%s'\n",stream->full_name,suggested_filename));
+	    if(rename(stream->full_name,dir_file(dir,suggested_filename))==-1)
+	       debug((3,"rename(%s, %s): %s\n",stream->full_name,suggested_filename,strerror(errno)));
+	 }
 	 done=true;
 	 m=MOVED;
       }
