@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ResMgr.h,v 1.39 2005/06/21 12:14:06 lav Exp $ */
+/* $Id: ResMgr.h,v 1.42 2005/12/27 12:02:39 lav Exp $ */
 
 #ifndef RESMGR_H
 #define RESMGR_H
@@ -194,6 +194,7 @@ class TimeInterval : public TimeDiff
    const char *error_text;
    void init(const char *);
 public:
+   void Set(const char *s) { init(s); }
    TimeInterval(const char *s) { init(s); }
    TimeInterval(ResValue r) { init(r); }
    TimeInterval(time_t i) : TimeDiff(i,0) { infty=false; error_text=0; }
@@ -206,16 +207,17 @@ public:
 
 class Range
 {
-   int start;
-   int end;
-   bool full;
+   long long start,end;
+   bool no_start,no_end;
    const char *error_text;
 
+   static const char *scale(long long *value,char suf);
+
 public:
    Range(const char *s);
-   bool Match(int n);
-   bool IsFull() { return full; }
-   int Random();
+   bool Match(long long n) const { return (no_start || n>=start) && (no_end || n<=end); }
+   bool IsFull() { return no_start && no_end; }
+   long long Random();
    bool Error() { return error_text!=0; };
    const char *ErrorText() { return error_text; }
 };
