@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2002 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2004 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: StatusLine.cc,v 1.24 2002/08/05 08:51:14 lav Exp $ */
+/* $Id: StatusLine.cc,v 1.33 2005/08/12 07:05:32 lav Exp $ */
 
 #include <config.h>
 #include <sys/types.h>
@@ -29,6 +29,8 @@
 #include <sys/ioctl.h>
 #include "trio.h"
 #include <stdarg.h>
+#include <stdlib.h>
+#include <mbswidth.h>
 #include "xstring.h"
 #include "xmalloc.h"
 
@@ -50,6 +52,7 @@ int  StatusLine::GetWidth()
       sz.ws_col=80;
    if(sz.ws_row==0)
       sz.ws_row=24;
+   LastHeight=sz.ws_row;
    return(LastWidth=sz.ws_col);
 #else /* !TIOCGWINSZ */
    return 80;
@@ -61,10 +64,9 @@ StatusLine::StatusLine(int new_fd)
    fd=new_fd;
    update_delayed=false;
    next_update_title_only=false;
-   strcpy(shown,"");
    strcpy(def_title,"");
    not_term=!isatty(fd);
-   LastWidth=GetWidth();
+   GetWidth();
    Reconfig();
 }
 
@@ -76,10 +78,9 @@ StatusLine::~StatusLine()
 
 void StatusLine::Clear(bool title_also)
 {
-   char newstr[sizeof(shown)];
-
-   newstr[0]=0;
-   update(newstr);
+   const char *empty="";
+   update_timer.Stop();
+   ShowN(&empty,1);
    update_delayed=false;
    update_timer.SetMilliSeconds(20);
 
@@ -101,30 +102,41 @@ void StatusLine::Show(const char *f,...)
       return;
    }
 
-   char newstr[sizeof(shown)];
+   char newstr[0x800];
 
    va_list v;
    va_start(v,f);
    vsnprintf(newstr,sizeof(newstr),f,v);
    va_end(v);
+   newstr[sizeof(newstr)-1]=0;
 
-   if(update_timer.Stopped())
+   const char *s=newstr;
+   ShowN(&s,1);
+}
+
+void StatusLine::ShowN(const char *const* newstr,int n)
+{
+   if(!update_delayed && shown.IsEqual(newstr,n))
+      return;
+   if(update_delayed && to_be_shown.IsEqual(newstr,n))
+      return;
+
+   if(!update_timer.Stopped())
    {
-      update(newstr);
+      /* not yet */
+      to_be_shown.Assign(newstr,n);
+      update_delayed=true;
+   }
+   else
+   {
+      update(newstr,n);
       update_delayed=false;
-      return;
    }
-
-   if(!strcmp(to_be_shown,newstr))
-      return;
-
-   /* not yet */
-   strcpy(to_be_shown,newstr);
-   update_delayed=true;
 }
 
 const char *StatusLine::to_status_line = get_string_term_cap("tsl", "ts");
 const char *StatusLine::from_status_line = get_string_term_cap("fsl", "fs");
+const char *StatusLine::prev_line = get_string_term_cap("cuu1","up");
 
 void StatusLine::WriteTitle(const char *s, int fd) const
 {
@@ -156,7 +168,7 @@ void StatusLine::WriteTitle(const char *s, int fd) const
    xfree(disp);
 }
 
-void StatusLine::update(char *newstr)
+void StatusLine::update(const char *const *newstr,int newstr_height)
 {
    if(not_term)
       return;
@@ -165,7 +177,7 @@ void StatusLine::update(char *newstr)
       return;
 
    /* Don't write blank titles into the title; let Clear() do that. */
-   if(newstr[0]) WriteTitle(newstr, fd);
+   if(newstr_height>0 && newstr[0][0]) WriteTitle(newstr[0], fd);
 
    if(next_update_title_only)
    {
@@ -173,81 +185,91 @@ void StatusLine::update(char *newstr)
       return;
    }
 
-   char *end=newstr+strlen(newstr);
-
    int w=GetWidth();
-   if(end-newstr>=w)
-      end=newstr+w-1;
+   int mbflags=MBSW_ACCEPT_INVALID|MBSW_ACCEPT_UNPRINTABLE;
 
-   while(end>newstr && end[-1]==' ')
-      end--;
+   if(newstr_height>LastHeight)
+      newstr_height=LastHeight;
 
-   *end=0;
+   // clear old extra lines. Assume we are at beginning of last shown line.
+   int j=shown.Count();
+   int i=j-newstr_height;
+   char *spaces=string_alloca(w+1);
+   memset(spaces,' ',w);
+   spaces[w]=0;
+   while(i-->0)
+   {
+      int tw=mbswidth(shown[--j],mbflags);
+      write(fd,"\r",1);
+      write(fd,spaces,tw);
+      write(fd,"\r",1);
+      write(fd,prev_line,strlen(prev_line));
+   }
+   // move to top of shown lines.
+   while(--j>0)
+      write(fd,prev_line,strlen(prev_line));
 
-   if(!strcmp(shown,newstr))
-      return;
+   int curr_line=0;
+   while(curr_line<newstr_height)
+   {
+      const char *end=newstr[curr_line];
+      int len=strlen(newstr[curr_line]);
+      int wpos=0;
+      while(len>0)
+      {
+	 int ch_len=mblen(end,len);
+	 if(ch_len<1)
+	    ch_len=1;
+	 int ch_width=mbsnwidth(end,ch_len,mbflags);
+	 if(wpos+ch_width>w-1)
+	    break;
+	 end+=ch_len;
+	 len-=ch_len;
+	 wpos+=ch_width;
+	 if(wpos>=w-1)
+	    break;
+      }
 
-   int dif=strlen(shown)-strlen(newstr)+2;
+      // FIXME: this assumes that multibyte chars cannot include ' '.
+      while(end>newstr[curr_line] && end[-1]==' ')
+      {
+	 end--;
+	 wpos--;  // FIXME: assumption - space width is 1
+      }
 
-   strcpy(shown,newstr);
+      if(end-newstr[curr_line]>0)
+	 write(fd,newstr[curr_line],end-newstr[curr_line]);
 
-   while(dif-->0 && end-newstr<w-1)
-      *end++=' ';
+      const char *shown_curr=(curr_line>=shown.Count()?"":shown[curr_line]);
+      int dif=strlen(shown_curr)-(end-newstr[curr_line])+2;
+      if(dif>(w-1)-wpos)
+	 dif=(w-1)-wpos;
+      if(dif>0)
+	 write(fd,spaces,dif);
+      write(fd,"\r",1);
 
-   *end=0;
+      if(++curr_line<newstr_height)
+	 write(fd,"\n",1);
+   }
 
-   if(end==newstr)
-      return;
-
-   *end++='\r';
-   *end=0;
-
-   write(fd,"\r",1);
-   write(fd,newstr,strlen(newstr));
+   shown.Assign(newstr,newstr_height);
 
    update_timer.SetResource("cmd:status-interval",0);
 }
 
 void StatusLine::WriteLine(const char *f,...)
 {
-   char *newstr=(char*)alloca(sizeof(shown)+strlen(f)+64);
+   char *newstr=string_alloca(0x800+strlen(f));
 
    va_list v;
    va_start(v,f);
    vsprintf(newstr,f,v);
    va_end(v);
 
-   if(not_term || shown[0]==0)
-   {
-      strcat(newstr,"\n");
-      write(fd,newstr,strlen(newstr));
-      update_delayed=false;
-      return;
-   }
+   Clear();
 
-   char *end=newstr+strlen(newstr);
-   while(end>newstr && end[-1]==' ')
-      end--;
-   *end=0;
-   if(!strcmp(shown,newstr))
-   {
-      write(fd,"\n",1);
-      return;
-   }
-
-   int dif=strlen(shown)-strlen(newstr)+2;
-   int w=GetWidth();
-
-   while(dif-->0 && end-newstr<w-1)
-      *end++=' ';
-
-   *end++='\n';
-   *end=0;
-
-   write(fd,"\r",1);
+   strcat(newstr,"\n");
    write(fd,newstr,strlen(newstr));
-
-   strcpy(shown,"");
    update_delayed=false;
 }
 
@@ -257,7 +279,7 @@ int StatusLine::Do()
       return STALL;
    if(update_timer.Stopped())
    {
-      update(to_be_shown);
+      update(to_be_shown.Set(),to_be_shown.Count());
       update_delayed=false;
       return STALL;
    }
