@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: keyvalue.cc,v 1.14 2003/12/17 13:00:04 lav Exp $ */
+/* $Id: keyvalue.cc,v 1.16 2005/12/31 07:01:38 lav Exp $ */
 
 #include <config.h>
 #include "keyvalue.h"
@@ -229,15 +229,18 @@ int KeyValueDB::Lock(int fd,int type)
    if(res==-1 && E_RETRY(errno))
    {
       int retries=5;
+      bool echo=true;
       for(int i=0; i<retries; i++)
       {
 	 sleep(1);
-	 write(2,".",1);
+	 if(echo && write(2,".",1)==-1)
+	    echo=false;
 	 res=fcntl(fd,F_SETLK,&lk);
 	 if(res==0)
 	    break;
       }
-      write(2,"\r",1);
+      if(echo && write(2,"\r",1)==-1)
+	 echo=false;
    }
    if(res==-1 && E_LOCK_IGNORE(errno))
       return 0;
