@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: misc.h,v 1.37 2005/04/28 22:59:39 lav Exp $ */
+/* $Id: misc.h,v 1.40 2005/12/27 12:23:19 lav Exp $ */
 
 #ifndef MISC_H
 #define MISC_H
@@ -36,10 +36,8 @@ const char *expand_home_relative(const char *);
 
 // returns ptr to last path element
 const char *basename_ptr(const char *);
-static inline
-char *basename_ptr(char *f)
-{
-   return (char*)basename_ptr((const char *)f);
+static inline char *basename_ptr(char *f) {
+   return const_cast<char*>(basename_ptr(const_cast<const char *>(f)));
 }
 
 // glues file to dir; returns pointer to static storage
@@ -121,4 +119,11 @@ void base64_encode (const char *s, char *store, int length);
 
 bool temporary_network_error(int e);
 
+CDECL const char *get_lftp_home();
+
+const char *memrchr(const char *buf,char c,size_t len);
+static inline char *memrchr(char *buf,char c,size_t len) {
+   return const_cast<char*>(memrchr(const_cast<const char*>(buf),c,len));
+}
+
 #endif // MISC_H
