@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: resource.cc,v 1.120 2005/07/04 13:55:40 lav Exp $ */
+/* $Id: resource.cc,v 1.123 2005/12/27 12:11:33 lav Exp $ */
 
 #include <config.h>
 
@@ -221,6 +221,7 @@ static ResDecl
    ResDecl11f("ftp:ssl-protect-list",	  "yes",   ResMgr::BoolValidate,0),
    ResDecl11d("ftp:ssl-auth",		  "TLS",   AuthArgValidate,0),
    ResDecl11e("ftp:ssl-allow-anonymous",  "no",	   ResMgr::BoolValidate,0),
+   ResDecl11i("ftp:ssl-use-ccc",	  "no",	   ResMgr::BoolValidate,0),
    ResDecl11h("ftps:initial-prot",	  "",	   ProtValidate,0),
 #endif
    ResDecl12 ("ftp:stat-interval",	  "1",	   ResMgr::UNumberValidate,0),
@@ -337,7 +338,8 @@ static ResDecl
    res_cache_enable("dns:cache-enable", "yes", ResMgr::BoolValidate,0),
    res_cache_expire("dns:cache-expire", "1h",  ResMgr::TimeIntervalValidate,0),
    res_cache_size  ("dns:cache-size",   "256", ResMgr::UNumberValidate,ResMgr::NoClosure),
-   res_timeout	   ("dns:fatal-timeout","0",   ResMgr::UNumberValidate,0),
+   res_timeout	   ("dns:fatal-timeout","7d",  ResMgr::TimeIntervalValidate,0),
+   res_max_retries ("dns:max-retries",  "1000",ResMgr::UNumberValidate,0),
    res_order	   ("dns:order",	DEFAULT_ORDER, OrderValidate,0),
    res_query_srv   ("dns:SRV-query",    "no",  ResMgr::BoolValidate,0),
    res_use_fork	   ("dns:use-fork",     "yes", ResMgr::BoolValidate,ResMgr::NoClosure);
@@ -350,6 +352,7 @@ static ResDecl
    res_dir_colors ("color:dir-colors",   "",   0,ResMgr::NoClosure);
 
 static ResDecl
+   res_dest_dir	  ("xfer:destination-directory","",0,0),
    res_verify	  ("xfer:verify", "no", ResMgr::BoolValidate,ResMgr::NoClosure),
    res_verify_cmd ("xfer:verify-command","",ResMgr::FileExecutable,0);
 
